require 'optparse'

require 'gill/helper'
require 'gill/config'
require 'gill/remove'
require 'gill/import'
require 'gill/parse'
require 'gill/git'
require 'gill/exceptions'

module Gill

  module CLI
    # Include The Gill Helper Methods
    include Helper

    #
    # CLI.Pull
    #
    # @param [String] args Repository, folder name and/or repository name.
    #
    def CLI.pull(args)
      begin
        require 'rubygems'
        require 'bundler'
        uri = Parse.new(args)
        git = Git.new(uri.category, uri.folder, uri.repo)
        git.clone
      rescue LoadError
        STDERR.puts "Count not load the bundler gem. Install it with 'gem install bundler'."
        exit -1
      rescue TypeError => e
        STDERR.puts "Error: #{e}"
      rescue => e
        STDERR.puts "Error: #{e}"
      end
    end

    #
    # Import any preexisting repos intop the gill cache.
    #
    def CLI.import
      begin
        search = Import.new
        search.import
        search.update_cache
        exit -1
      rescue => e
        STDERR.puts "Error: #{e}"
        exit -1
      end
    end

    #
    # Update a gill repository.
    #
    def CLI.update(repo_name)
      begin
        git = Git.new(false, repo_name, false)
        git.update
        exit -1
      rescue TypeError => e
        STDERR.puts "Error: #{e}\n"
        exit -1
      rescue => e
        STDERR.puts "Error: #{e}"
        exit -1
      end
    end

    #
    # List all gill cloned repositories.
    #
    def CLI.list
      Gill.config.list_entries
      exit -1
    end

    #
    # Remove a gill cloned repository.
    #
    # @param [String] args The category and repository name to remove.
    #
    def CLI.remove(repo_name)
      begin

      Gill.config.find_repo_paths_by_name(repo_name, false) do |repo,path,category|
        Remove.new("#{category}##{repo}").remove_repo
      end

      rescue => e
        STDERR.puts "Error: #{e}"
        exit -1
      end
      exit -1
    end

    # Gill option parser
    # @param [Array] passed gill options.
    def optparse(*args)

      @opts = OptionParser.new

      @options = {}

      @opts.banner = "Gill - Git, Clone, Cleanliness.\nUsage: gill git://host/repository.git#category/sub-category/...\n\n"

      @opts.on('-l','--list','List all cloned repositories.') do |list|
        CLI.list
      end

      @opts.on('-p ','--path ','Ouput the path of a repository. e.g. --path gill') do |repo_name|
        Gill.config.find_repo_paths_by_name(repo_name)
        exit -1
      end

      @opts.on('-r ','--remove ','Remove repository. e.g. -r category#repository') do |remove|
        CLI.remove(remove)
      end

      @opts.on('-u ', '--update ','Update repository. e.g. -u *gill*') do |update|
        CLI.update(update)
      end

      @opts.on('-i','--import','Import untracked repositories.') do |import|
        CLI.import
      end

      @opts.on('-h','--help','This help summary page.') do |help|
        usage
      end

      @opts.on('-v','--version','Version number') do |version|
        STDOUT.puts "Version: #{Gill::VERSION}"
        exit -1
      end

      begin
        @args = @opts.parse!(args)
        usage if @args.empty? || @args[0].nil?
        CLI.pull(@args[0])
      rescue Interrupt
        STDERR.puts "\nExiting..."
      rescue OptionParser::MissingArgument => e
        STDERR.puts e.message
        STDOUT.puts @opts
        STDOUT.puts "\n"
        exit -1
      rescue OptionParser::InvalidOption => e
        STDERR.puts e.message
        STDOUT.puts @opts
        STDOUT.puts "\n"
        exit -1
      end
    end

    #
    # Gill usage.
    # @return [String] Gill Usage Information.
    #
    def usage
      STDOUT.puts "#{@opts}\n"
      exit -1
    end

  end

end
