module Gill

  module Settings

    class Config
      include Helper

      attr_reader :home, :config_path, :cache_path, :settings, :cache, :repos

      #
      # Initialize Gill Configuration
      # 
      # @param [String] home_path Override the default user home path.
      #
      def initialize(home_path=nil)
        @home = home_path || File.expand_path('~')
        @settings_path = File.join(@home, '.gillrc')
        @cache_path = File.join(@home, '.gillcache')
        Dir.chdir(@home)
        file_check!
        version_check!
        check_default_folders!
        self
      end

      #
      # Gill File Check
      #
      def file_check!
        if File.exists?(@settings_path)
          load_gill_settings
        else
          setup_gill_settings
        end
        if File.exists?(@cache_path)
          load_gill_cache
        else
          setup_gill_cache
        end
      end

      #
      # Gill Categories
      # 
      # @return [Hash] Gill Category Hash
      #
      def categories
        @repos = @cache[:categories]
      end

      #
      # Gill Source Path
      # 
      # @return [String] Source Path
      #
      def source_path
        @source_path = @settings[:source]
      end

      #
      # Cache Path
      # 
      # @return [String] Gill Cache Path
      #
      def cache_path
        @source_path = @settings[:cache]
      end

      #
      # Gill Settings Path
      # 
      # @return [String] Gill Settings Path
      #
      def settings_path
        @settings_path = @settings[:settings]
      end

      #
      # Gill Default Path
      # 
      # @return [String] Gill Default Clone Path
      #
      def default_path
        @default_path = @settings[:default]
      end

      #
      # Write Settings
      # 
      # @param [Hash] settings Hash To Write To The Gill Settings
      #
      def write_settings(settings)
        File.open(@settings_path, 'w') do |out|
          YAML.dump(settings, out)
        end
      end

      #
      # Write Cache
      # 
      # @param [Hash] cache Hash To Write To The Gill Cache
      #
      def write_cache(cache)
        File.open(@cache_path, 'w') do |out|
          YAML.dump(cache, out)
        end
      end

      #
      # List Gill Cloned Repos
      # 
      #
      def list_entries
        STDERR.puts green("\n*** Listing Cloned Repositories ***\n")
        categories.sort.each do |category,repos|
          STDERR.puts "\t#{category}:"
          repos.each do |repo|
            STDERR.puts "\t  #{green(repo[0])} => " + "#{repo[1][:repo]}"
          end
          STDERR.puts "\n"
        end
      end

      #
      # Clean Cache
      # 
      # Check the gill cache on load for missing
      # values and remove.
      #
      def clean_cache
        @clean_cache = Gill.config.cache
        
        @clean_cache[:categories].each do |key,value|
          value.each do |k,v|
            @clean_cache[:categories][key.to_sym].delete(k.to_sym) unless File.directory?(v[:path].to_s)
          end
          @clean_cache[:categories].delete(key.to_sym) if @clean_cache[:categories][key.to_sym].empty?
        end
        
        write_cache(@clean_cache)
      end

      #
      # Gill Paths
      # 
      # @return [Array] Array Of Gill Repo Paths
      #
      def paths
        @paths = []
        Gill.config.categories.each do |key,value|
          value.each do |k,v|
            @paths << v[:path]
          end
        end
        @paths
      end
      
      #
      # Find Repo
      # 
      # @param [String] repo_name Repository Name
      # 
      # @return [String] Repository Path 
      #
      def find_repo_paths_by_name(repo_name,pretty_print=true,&block)
        repos = Dir[File.join(Gill.config.source_path, '**', "*#{repo_name}*")]
        repos.each do |path|
          next unless Gill.config.paths.include?(path)
          yield [basename(path),path,"#{basename(parent(path))}".downcase] if block && !pretty_print
          STDOUT.puts "#{basename(path)} => #{green(path)}" if pretty_print
        end
      end

      private

        def version_check!
          unless current_settings_version && current_cache_version
            update_gill_settings
            update_gill_cache
          end
        end

        def update_gill_cache
          update_cache = @cache
          update_cache[:version] = VERSION
          write_cache(update_cache)
        end
        
        def update_gill_settings
          update_settings = @settings
          update_settings[:version] = VERSION
          write_settings(update_settings)
        end

        def remove_gill_configuration_files
          FileUtils.rm_rf(@cache_path)
          FileUtils.rm_rf(@settings_path)
        end

        def current_settings_version
          return true if (@settings[:version] == Gill::VERSION)
          false
        end

        def current_cache_version
          return true if (@cache[:version] == Gill::VERSION)
          false
        end

        def load_gill_settings
          @settings = YAML.load_file(@settings_path)
        end

        def load_gill_cache
          @cache = YAML.load_file(@cache_path)
        end

        def setup_gill_settings
          settings = {
            :version => Gill::VERSION,
            :settings => @settings_path,
            :source => "#{@home}/source",
            :default => "#{@home}/source/misc",
            :cache => @cache_path
          }
          write_settings(settings)
          file_check!
        end

        def setup_gill_cache
          cache = { :version => Gill::VERSION, :categories => {} }
          write_cache(cache)
          file_check!
        end

        def check_default_folders!
          FileUtils.mkdir_p(source_path)
          FileUtils.mkdir_p(default_path)
        end

    end
  end
end
