module Gill

  #
  # GIT ENV Variables that interfere with paths
  #
  ENV_BLACKLIST = %w(GIT_DIR GIT_INDEX_FILE GIT_WORK_TREE)

  class Git
    include Helper
    include Settings

    attr_reader :category, :category_path, :repo_name, :repo_path

    #
    # Initialize New Git Object
    # 
    # @param [String] category Repo Category
    # 
    # @param [String] repo_name Repo Name
    # 
    # @param [String] repo_uri Repo URI
    #
    def initialize(category,repo_name,repo_uri)

      @repo_uri, @category, @repo_name = repo_uri, category, repo_name

      @category ||= File.basename(Gill.config.default_path)
      @category_path = File.join(Gill.config.source_path, @category)

      if @repo_name
        @repo_path = File.join(@category_path, @repo_name)
      else
        @repo_path = @category_path
      end
    end

    #
    # Update Cache
    # 
    # Update the gill cache of a successful clone 
    # 
    # and write changes to the Gill global cache.
    #
    def update_cache
      cache = Gill.config.cache

      cache[:categories][@category.to_sym] ||= {}
      cache[:categories][@category.to_sym].merge!(
        @repo_name.to_sym => {
        :repo => @repo_uri,
        :path => @repo_path
        }
      )

      Gill.config.write_cache(cache)
    end

    #
    # Create the repository category folder if it does not exist.
    #
    # @param [Boolean] make_folder
    #   Pass false to ignore the creation of the passed gill folder
    #
    def build_category_folder
      FileUtils.mkdir_p(@category_path)
    end

    #
    # Bundle
    # 
    # @param [String] path Repository Path
    #
    def bundle_from_gemfile(path)
      Dir.chdir(path) do
        return unless File.exists?('Gemfile')
        ask "Would you like to `bundle install`?", :default => true do
          system('bundle install')
        end
      end
    end

    #
    # Update/Bundle Repository
    #
    def update
      paths = Dir[File.join(Gill.config.source_path, '**', "*#{@repo_name}*")]      
      paths.each do |path|
        if Gill.config.paths.include?(path)
          ask "Update #{basename(path)} => #{green(path)}?", :default => true do
            Dir.chdir(path) do
              system('git', 'pull', 'origin', 'master')
              bundle_from_gemfile(path)
            end
          end
        end
      end
    end

    #
    # Clone the passed repository uri name.
    #
    def clone(bundle_install=true)
      raise(DirectoryError, "destination path '#{@repo_path}' already exists and is not an empty directory.") if File.directory?(@repo_path)
      build_category_folder
      ENV_BLACKLIST.each(&(ENV.method(:delete)))
      update_cache if system('git', 'clone', @repo_uri, @repo_path)
      bundle_from_gemfile(@repo_path) if bundle_install
    end


  end
end
