require 'pathname'

module Gill

  class Import
    # Include the gill cache convenience methods.
    include Helper
    include Settings

    def initialize
      @updated_cache = Gill.config.cache
      STDOUT.puts "Please wait..."
    end

    #
    # Import
    # 
    # @param [String] path Override Default Import Path
    #
    def import(path=nil)

      if path.nil?
        folder = Gill.config.source_path
      else
        folder = path
      end

      search_folder(folder)
    end

    #
    # Find Repo
    # 
    # @param [String] path Folder Path
    # 
    # @param [String] repo_name Repository Name
    # 
    # @param [Object] parent Pathname Parent Pbject
    #
    def find_repo(path,repo_name,parent)

      if has_git_folder?(path)
        check_move_import(basename(parent.path), repo_name, path) if has_git_config?(path)
      else
        import(path)
      end

    end

    #
    # Search Folder
    # 
    # @param [String] folder Folder Path
    #
    def search_folder(folder)
      current_folder = Dir.new(folder)
      
      current_folder.entries.each do |file|
        next if %w(. ..).include?(file)
        path = File.join(current_folder.path, file)
        find_repo(path,file,current_folder) if File.directory?(path)
      end

    end

    #
    # Check, Move & import
    # 
    # @param [String] basename Basename Of The Repository Path
    # 
    # @param [String] repo_name Repository Name
    # 
    # @param [Object] path Pathname Parent Object
    #
    def check_move_import(basename,repo_name,path)
      git_uri = git_uri_for(path)

      if repo_in_root(basename)
        prompt = "Do you want to move #{basename(Gill.config.source_path)}/#{basename(repo_name)} to",
        "#{basename(Gill.config.source_path)}/#{basename(Gill.config.default_path)}/#{basename(repo_name)}?"
        
        ask prompt, :default => false do
          move_root_repos_to_default_path(path,File.join(Gill.config.source_path, basename(Gill.config.default_path), repo_name))
          add(basename(Gill.config.default_path),repo_name,git_uri,
          File.join(Gill.config.source_path,
          basename(Gill.config.default_path), 
          repo_name))
        end
        
      else
        add(basename,repo_name,git_uri,path)
      end

    end

    #
    # Has Git?
    # 
    # @param [String] path Folder Path
    # 
    # @return [true,false] Retrun true if the .git folder is found.
    #
    def has_git_config?(path)
      return true if File.exist?(File.join(path, '.git', 'config'))
      false
    end

    #
    # Move Repository
    # 
    # @param [String] source Repository Source Path
    # @param [String] destination Repository Destination Path
    #
    def move_root_repos_to_default_path(source,destination)
      FileUtils.mv(source,destination)
    end

    #
    # Git URI For Path
    # 
    # @param [String] path Repository Path
    # 
    # @return [String] Return the git uri for a path.
    #
    def git_uri_for(path)
      File.open(File.join(path, '.git', 'config'), 'r') do |file|
        @git_uri = file.read[/url\s+\=\s+(\S+)/m,1]
        file.close
      end
      @git_uri
    end

    #
    # Add Repository
    # 
    # @param [String] category Repository Category
    # 
    # @param [String] repo_name Repository Name
    # 
    # @param [String] git_uri Repository URI
    # 
    # @param [String] path Repository Path
    #
    def add(category,repo_name,git_uri,path)
      if @updated_cache[:categories].has_key?(category.to_s.downcase.to_sym)
        @updated_cache[:categories][category.to_s.downcase.to_sym].merge!({repo_name.to_sym => {:repo => git_uri, :path => path}})
      else
        @updated_cache[:categories][category.to_s.downcase.to_sym] = { repo_name.to_sym => { :repo => git_uri, :path => path }}
      end
    end

    def update_cache
      STDOUT.puts green("Import Completed Successfully.")
      Gill.config.write_cache(@updated_cache)
    end

    #
    # Repository In Root
    # 
    # @param [String] folder Folder Path
    # 
    # @return [true,false] Return true if the Repository
    # is in the root dir. 
    #
    def repo_in_root(folder)
      return true if folder.to_s == basename(Gill.config.source_path).to_s
      false
    end

    #
    # Has Git Folder
    # 
    # @param [String] path Folder Path
    # 
    # @return [true,false] Return true if the folder
    # path contains a .git folder.
    #
    def has_git_folder?(path)
      temp = Dir.new(path)

      if temp.entries.include?('.git')
        temp.close
        return true
      else
        return false
      end

    end

  end
end
